#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025


"""This module contains the UserAPIClient class."""

import requests
from ibm_watsonx_data_integration.cpd_api.base import BaseAPIClient
from typing import TYPE_CHECKING
from urllib.parse import quote

if TYPE_CHECKING:
    from ibm_watsonx_data_integration.common.auth import BaseAuthenticator

DEFAULT_USER_API_VERSION = 2


class UserAPIClient(BaseAPIClient):
    """The API client of the User service."""

    def __init__(
        self,
        auth: "BaseAuthenticator",  # pragma: allowlist secret
        base_url: str = "https://user-management.cloud.ibm.com",
    ) -> None:
        """Initializes the UserAPIClient.

        Args:
            auth: The Authentication object.
            base_url: The User Service URL.
        """
        super().__init__(auth=auth, base_url=base_url)
        self.url_path = f"v{DEFAULT_USER_API_VERSION}/accounts"

    def get_users(self, account_id: str, params: dict = None) -> requests.Response:
        """Retrieve a list of users in the account by the account ID.

        Args:
            account_id: Unique identifier of the account.
            params: Query Parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(account_id, safe='')}/users"
        response = self.get(url=url, params=params)
        return response

    def get_user_profile(self, account_id: str, iam_id: str) -> requests.Response:
        """Retrieve a user's profile by the user's IAM ID in the account.

        Args:
            account_id: Unique identifier of the account.
            iam_id: Unique identifier of the user.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(account_id, safe='')}/users/{quote(iam_id, safe='')}"
        response = self.get(url=url)
        return response

    def delete_user_from_account(self, account_id: str, iam_id: str) -> requests.Response:
        """Remove a user from an account by user's IAM ID.

        Args:
            account_id: Unique identifier of the account.
            iam_id: Unique identifier of the user.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(account_id, safe='')}/users/{quote(iam_id, safe='')}"
        response = self.delete(url=url)
        return response

    def delete_user_from_account_by_id_or_email(self, account_id: str, params: dict) -> requests.Response:
        """Remove a user from an account by user's user ID or email.

        Args:
            account_id: Unique identifier of the account.
            params: Query parameters.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(account_id, safe='')}/users"
        response = self.delete(url=url, params=params)
        return response

    def get_user_settings(self, account_id: str, iam_id: str) -> requests.Response:
        """Retrieve a user's settings by the user's IAM ID in the account.

        Args:
            account_id: Unique identifier of the account.
            iam_id: Unique identifier of the user.

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(account_id, safe='')}/users/{quote(iam_id, safe='')}/settings"
        response = self.get(url=url)
        return response

    def update_user_settings(self, account_id: str, iam_id: str, data: dict) -> requests.Response:
        """Update the settings for a specific user in an account.

        Args:
            account_id: Unique identifier of the account.
            iam_id: Unique identifier (IAM ID) of the user.
            data: Dictionary containing any or all settings to update. For example:
                {
                    "language": "en-us",
                    "notification_language": "en-us",
                    "allowed_ip_addresses": "32.96.110.50,172.16.254.1",
                    "self_manage": True,
                    "2FA": False,
                    "security_questions_required": False,
                    "security_questions_setup": False
                }

        Returns:
            A HTTP response.
        """
        url = f"{self.base_url}/{self.url_path}/{quote(account_id, safe='')}/users/{quote(iam_id, safe='')}/settings"
        response = self.patch(url=url, data=data)
        return response



#  IBM Confidential
#  PID 5900-BAF
#  Copyright StreamSets Inc., an IBM Company 2025

"""The __init__ for the data module."""

from ibm_watsonx_data_integration.services.streamsets.data.pipeline_definition import (
    create_pipeline_definition_for_new_flow,
)

__all__ = ["create_pipeline_definition_for_new_flow"]



from ibm_watsonx_data_integration.services.datastage.models.enums import ORACLE_DATASTAGE_CONNECTION
from ibm_watsonx_data_integration.services.datastage.models.flow.datastage_flow import DataStageFlow
from ibm_watsonx_data_integration.services.datastage.tests.integration.util import AbstractFlowRunTest
from os import environ
from ibm_watsonx_data_integration.cpd_models.project_model import Project


class TestOracleDataStageConnection(AbstractFlowRunTest):
    def get_flow_name(self):
        return "oracleDS_peek"

    def get_fc(self, project: Project):
        fc = DataStageFlow(project=project, name=self.get_flow_name(), description="", flow_type="datastage")

        # Stages
        _mytesttable1_1_ = fc.add_stage("Oracle Database for DataStage", "mytesttable1_1")
        _mytesttable1_1_.configuration.generate_sql_at_runtime = True
        _mytesttable1_1_.configuration.table_name = '"TM_DS"."mytesttable1"'

        _mytesttable1_1_.configuration.connection.cas_lite_service_authorization_header = ""
        _mytesttable1_1_.configuration.connection.connection_type = ORACLE_DATASTAGE_CONNECTION.ConnectionType.tcp
        _mytesttable1_1_.configuration.connection.hostname = environ["ORACLEDS_HOST"]
        _mytesttable1_1_.configuration.connection.port = "1521"
        _mytesttable1_1_.configuration.connection.servicename = "orclpdb"
        _mytesttable1_1_.configuration.connection.password = environ["ORACLEDS_PASSWORD"]
        _mytesttable1_1_.configuration.connection.use_connection_string = False
        _mytesttable1_1_.configuration.connection.username = environ["ORACLEDS_USERNAME"]
        _mytesttable1_1_.configuration.connection.gateway_url = "https://internal-nginx-svc.ds.svc:12443"

        _peek_2_ = fc.add_stage("Peek", "Peek_2")

        # Graph
        mytesttable1_1_schema = _mytesttable1_1_.connect_output_to(_peek_2_).set_name("Link_2").create_schema()

        # Schemas
        mytesttable1_1_schema.add_field("DECIMAL", "col1").nullable().length(38).precision(38)

        project.update_flow(fc)

        return fc

    def validate(self, state):
        assert "Completed" in state



from io import BytesIO
from pathlib import Path
from zipfile import ZipFile, ZipInfo


def _get_json_type(path: str):
    if path.startswith("data_intg_subflow"):
        return "subflow"
    elif path.startswith("data_intg_flow"):
        return "flow"
    elif path.startswith("parameter_set"):
        return "parameter_set"
    elif path.startswith("data_definition"):
        return "data_definition"
    elif path.startswith("data_intg_message_handler"):
        return "message_handler"
    elif path.startswith("data_intg_java_library"):
        return "java_library"
    elif path.startswith("connection"):
        return "connection"
    elif path.startswith("data_intg_parallel_function"):
        return "function_library"
    elif path.startswith("data_intg_build_stage"):
        return "build_stage"
    elif path.startswith("data_intg_wrapped_stage"):
        return "wrapped_stage"
    elif path.startswith("data_intg_custom_stage"):
        return "custom_stage"
    elif path.startswith("custom_stage_library"):
        return "custom_stage_attachment"
    elif path.startswith("ds_match_specification"):
        return "match_specification"
    elif path.startswith("job"):
        return "job"
    elif path.startswith("px_executables"):
        return "executables"
    elif path.startswith("ENCRYPTED"):
        return "encrypted"
    elif path.startswith("data_intg_test_case"):
        return "test_case"
    else:
        return ""


class ZipImporter:
    def __init__(self, file_path: str):
        self.file_path = file_path
        self.subflows: list[tuple[ZipInfo, bytes]] = []
        self.flows: list[tuple[ZipInfo, bytes]] = []
        self.paramsets: list[tuple[ZipInfo, bytes]] = []
        self.data_definitions: list[tuple[ZipInfo, bytes]] = []
        self.message_handlers: list[tuple[ZipInfo, bytes]] = []
        self.java_libraries: list[tuple[ZipInfo, bytes]] = []
        self.connections: list[tuple[ZipInfo, bytes]] = []
        self.function_libraries: list[tuple[ZipInfo, bytes]] = []
        self.build_stages: list[tuple[ZipInfo, bytes]] = []
        self.wrapped_stages: list[tuple[ZipInfo, bytes]] = []
        self.custom_stages: list[tuple[ZipInfo, bytes]] = []
        self.custom_stage_attachments: list[tuple[ZipInfo, bytes]] = []
        self.match_specifications: list[tuple[ZipInfo, bytes]] = []
        self.jobs: list[tuple[ZipInfo, bytes]] = []
        self.executables: list[tuple[ZipInfo, bytes]] = []
        self.encrypted: list[tuple[ZipInfo, bytes]] = []
        self.test_cases: list[tuple[ZipInfo, bytes]] = []

    def find_paramset(self, name: str) -> bytes:
        for f_info, f_content in self.paramsets:
            if Path(f_info.filename).stem == name:
                return f_content

        raise KeyError(f"Parameter set '{name}' not found in the zip file.")

    def find_subflow(self, name: str) -> bytes:
        for f_info, f_content in self.subflows:
            if Path(f_info.filename).stem == name:
                return f_content

        raise KeyError(f"Subflow '{name}' not found in the zip file.")

    def find_build_stage(self, name: str) -> bytes:
        for f_info, f_content in self.build_stages:
            if Path(f_info.filename).stem == name:
                return f_content

        raise KeyError(f"Build stage '{name}' not found in the zip file.")

    def find_wrapped_stage(self, name: str) -> bytes:
        for f_info, f_content in self.wrapped_stages:
            if Path(f_info.filename).stem == name:
                return f_content

        raise KeyError(f"Wrapped stage '{name}' not found in the zip file.")

    def find_custom_stage(self, name: str) -> bytes:
        for f_info, f_content in self.custom_stages:
            if Path(f_info.filename).stem == name:
                return f_content

        raise KeyError(f"Custom stage '{name}' not found in the zip file.")

    def find_custom_stage_attachment(self, name: str) -> tuple[ZipInfo, bytes]:
        for f_info, f_content in self.custom_stage_attachments:
            if Path(f_info.filename).stem == name:
                return (f_info, f_content)

        raise KeyError(f"Custom stage attachment'{name}' not found in the zip file.")

    def run(self):
        with ZipFile(self.file_path) as zip_file:
            for f_info in zip_file.infolist():
                f_type = _get_json_type(f_info.filename)
                f_content = zip_file.read(f_info.filename)

                match f_type:
                    case "subflow":
                        self.subflows.append((f_info, f_content))
                    case "flow":
                        self.flows.append((f_info, f_content))
                    case "parameter_set":
                        self.paramsets.append((f_info, f_content))
                    case "data_definition":
                        self.data_definitions.append((f_info, f_content))
                    case "message_handler":
                        self.message_handlers.append((f_info, f_content))
                    case "java_library":
                        self.java_libraries.append((f_info, f_content))
                    case "connection":
                        self.connections.append((f_info, f_content))
                    case "function_library":
                        self.function_libraries.append((f_info, f_content))
                    case "build_stage":
                        self.build_stages.append((f_info, f_content))
                    case "wrapped_stage":
                        self.wrapped_stages.append((f_info, f_content))
                    case "custom_stage":
                        self.custom_stages.append((f_info, f_content))
                    case "custom_stage_attachment":
                        self.custom_stage_attachments.append((f_info, f_content))
                    case "match_specification":
                        self.match_specifications.append((f_info, f_content))
                    case "job":
                        self.jobs.append((f_info, f_content))
                    case "executables":
                        self.executables.append((f_info, f_content))
                    case "encrypted":
                        self.encrypted.append((f_info, f_content))
                    case "test_case":
                        with ZipFile(BytesIO(f_content)) as tc_zip:
                            for tc_f_info in tc_zip.infolist():
                                tc_f_content = tc_zip.read(tc_f_info.filename)
                                if tc_f_info.filename == "data_intg_test_case_metadata":
                                    self.test_cases.append((tc_f_info, tc_f_content))
                                else:
                                    with ZipFile(BytesIO(tc_f_content)) as csv_zip:
                                        for file_name in csv_zip.namelist():
                                            csv_zip.extract(
                                                path="test_case_data_files/",
                                                member=file_name,
                                            )  # Code to extract input and output csv files
                    case _:
                        pass



from enum import Enum
from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from ibm_watsonx_data_integration.services.datastage.models.enums import ODBC_CONNECTION
from pydantic import ConfigDict, Field
from typing import ClassVar


class AuthenticationMethod(Enum):
    oauth2 = "oauth2"
    serviceaccount = "serviceaccount"


class DsnType(Enum):
    Cassandra = "Cassandra"
    Hive = "Hive"
    Googlebigquery = "Googlebigquery"
    GreenPlum = "GreenPlum"
    DB2 = "DB2"
    DB2zOS = "DB2zOS"
    DB2AS400 = "DB2AS400"
    Informix = "Informix"
    Netezza = "Netezza"
    Impala = "Impala"
    MicrosoftSQLServer = "MicrosoftSQLServer"
    MongoDB = "MongoDB"
    MySQL = "MySQL"
    Oracle = "Oracle"
    PostgreSQL = "PostgreSQL"
    SybaseASE = "SybaseASE"
    SybaseIQ = "SybaseIQ"
    Teradata = "Teradata"
    Text = "Text"
    use_dsn_name = "use_dsn_name"


class LogOnMech(Enum):
    ldap = "ldap"
    td2 = "td2"


class ServiceAccountKeyInputMethod(Enum):
    keycontent = "keycontent"
    keyfile = "keyfile"


class OdbcConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "0ca92c3d-0e46-3b42-a573-77958d53c9be"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    access_token: str = Field(None, alias="access_token")
    additional_properties: str | None = Field(None, alias="additional_props")
    authentication_method: ODBC_CONNECTION.AuthenticationMethod = Field(
        ODBC_CONNECTION.AuthenticationMethod.oauth2, alias="authentication_method"
    )
    cas_lite_service_authorization_header: str | None = Field(None, alias="cas_lite_auth_header")
    client_id: str = Field(None, alias="client_id")
    client_secret: str = Field(None, alias="client_secret")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_nodes: str = Field(None, alias="cluster_nodes")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    database: str = Field(None, alias="database")
    dataset: str = Field(None, alias="dataset_name")
    default_port: int | None = Field(2638, alias="default_port")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    data_source_name: str = Field(None, alias="dsn_name")
    data_source_type: ODBC_CONNECTION.DsnType = Field(ODBC_CONNECTION.DsnType.DB2, alias="dsn_type")
    delimiter: str | None = Field(",", alias="host_port_separator")
    hostname_or_ip_address: str = Field(None, alias="hostname")
    keyspace: str | None = Field(None, alias="keyspace")
    log_on_mech: ODBC_CONNECTION.LogOnMech = Field(ODBC_CONNECTION.LogOnMech.td2, alias="log_on_mech")
    network_address: str = Field(None, alias="network_address")
    password: str | None = Field(None, alias="password")
    port: int = Field(None, alias="port")
    project: str = Field(None, alias="project_name")
    refresh_token: str = Field(None, alias="refresh_token")
    service_account_email: str = Field(None, alias="service_account_email")
    service_account_key_content: str = Field(None, alias="service_account_key_content")
    input_method_for_service_account_key: ODBC_CONNECTION.ServiceAccountKeyInputMethod = Field(
        ODBC_CONNECTION.ServiceAccountKeyInputMethod.keycontent, alias="service_account_key_input_method"
    )
    service_account_private_key_file_path: str | None = Field(None, alias="service_account_private_key")
    service_name: str = Field(None, alias="service_name")
    satellite_client_certificate: str | None = Field(None, alias="sl_client_cert")
    satellite_client_private_key: str | None = Field(None, alias="sl_client_private_key")
    satellite_connector_id: str | None = Field(None, alias="sl_connector_id")
    satellite_endpoint_host: str | None = Field(None, alias="sl_endpoint_host")
    satellite_endpoint_display_name: str | None = Field(None, alias="sl_endpoint_name")
    satellite_endpoint_port: int | None = Field(None, alias="sl_endpoint_port")
    original_hostname_of_the_resource: str | None = Field(None, alias="sl_host_original")
    satellite_as_http_proxy: bool | None = Field(None, alias="sl_http_proxy")
    satellite_location_id: str | None = Field(None, alias="sl_location_id")
    satellite_service_url: str | None = Field(None, alias="sl_service_url")
    use_cas_lite_service: bool | None = Field(True, alias="use_cas_lite")
    username: str | None = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        (
            include.add("log_on_mech")
            if (
                self.data_source_type
                and (
                    (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Teradata")
                    or (self.data_source_type == "Teradata")
                )
            )
            else exclude.add("log_on_mech")
        )
        (
            include.add("default_port")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                            or (self.data_source_type == "SybaseASE")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
            )
            else exclude.add("default_port")
        )
        (
            include.add("project")
            if (
                self.data_source_type
                and (
                    (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Googlebigquery")
                    or (self.data_source_type == "Googlebigquery")
                )
            )
            else exclude.add("project")
        )
        (
            include.add("data_source_name")
            if (
                self.data_source_type
                and (
                    (hasattr(self.data_source_type, "value") and self.data_source_type.value == "use_dsn_name")
                    or (self.data_source_type == "use_dsn_name")
                )
            )
            else exclude.add("data_source_name")
        )
        (
            include.add("client_id")
            if (
                (
                    (
                        (
                            self.data_source_type
                            and (
                                (
                                    hasattr(self.data_source_type, "value")
                                    and self.data_source_type.value == "Googlebigquery"
                                )
                                or (self.data_source_type == "Googlebigquery")
                            )
                        )
                    )
                )
                and (
                    self.authentication_method
                    and (
                        (hasattr(self.authentication_method, "value") and self.authentication_method.value == "oauth2")
                        or (self.authentication_method == "oauth2")
                    )
                )
            )
            else exclude.add("client_id")
        )
        (
            include.add("authentication_method")
            if (
                self.data_source_type
                and (
                    (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Googlebigquery")
                    or (self.data_source_type == "Googlebigquery")
                )
            )
            else exclude.add("authentication_method")
        )
        (
            include.add("password")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                            or (self.data_source_type == "Cassandra")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                        or (self.data_source_type == "DB2")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                        or (self.data_source_type == "Oracle")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                        or (self.data_source_type == "SybaseASE")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Teradata")
                        or (self.data_source_type == "Teradata")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "use_dsn_name")
                        or (self.data_source_type == "use_dsn_name")
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("hostname_or_ip_address")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                            or (self.data_source_type == "Cassandra")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                        or (self.data_source_type == "DB2")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                        or (self.data_source_type == "Oracle")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Teradata")
                        or (self.data_source_type == "Teradata")
                    )
                )
            )
            else exclude.add("hostname_or_ip_address")
        )
        (
            include.add("database")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                            or (self.data_source_type == "DB2")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                        or (self.data_source_type == "SybaseASE")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Teradata")
                        or (self.data_source_type == "Teradata")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Text")
                        or (self.data_source_type == "Text")
                    )
                )
            )
            else exclude.add("database")
        )
        (
            include.add("use_cas_lite_service")
            if (self.cas_lite_service_authorization_header)
            else exclude.add("use_cas_lite_service")
        )
        (
            include.add("client_secret")
            if (
                (
                    (
                        (
                            self.data_source_type
                            and (
                                (
                                    hasattr(self.data_source_type, "value")
                                    and self.data_source_type.value == "Googlebigquery"
                                )
                                or (self.data_source_type == "Googlebigquery")
                            )
                        )
                    )
                )
                and (
                    self.authentication_method
                    and (
                        (hasattr(self.authentication_method, "value") and self.authentication_method.value == "oauth2")
                        or (self.authentication_method == "oauth2")
                    )
                )
            )
            else exclude.add("client_secret")
        )
        (
            include.add("service_name")
            if (
                self.data_source_type
                and (
                    (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                    or (self.data_source_type == "Oracle")
                )
            )
            else exclude.add("service_name")
        )
        (
            include.add("cas_lite_service_authorization_header")
            if (self.use_cas_lite_service)
            else exclude.add("cas_lite_service_authorization_header")
        )
        (
            include.add("dataset")
            if (
                self.data_source_type
                and (
                    (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Googlebigquery")
                    or (self.data_source_type == "Googlebigquery")
                )
            )
            else exclude.add("dataset")
        )
        (
            include.add("service_account_private_key_file_path")
            if (
                (
                    (
                        (
                            self.data_source_type
                            and (
                                (
                                    hasattr(self.data_source_type, "value")
                                    and self.data_source_type.value == "Googlebigquery"
                                )
                                or (self.data_source_type == "Googlebigquery")
                            )
                        )
                    )
                )
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "serviceaccount"
                        )
                        or (self.authentication_method == "serviceaccount")
                    )
                )
                and (
                    self.input_method_for_service_account_key
                    and (
                        (
                            hasattr(self.input_method_for_service_account_key, "value")
                            and self.input_method_for_service_account_key.value == "keyfile"
                        )
                        or (self.input_method_for_service_account_key == "keyfile")
                    )
                )
            )
            else exclude.add("service_account_private_key_file_path")
        )
        (
            include.add("service_account_key_content")
            if (
                (
                    (
                        (
                            self.data_source_type
                            and (
                                (
                                    hasattr(self.data_source_type, "value")
                                    and self.data_source_type.value == "Googlebigquery"
                                )
                                or (self.data_source_type == "Googlebigquery")
                            )
                        )
                    )
                )
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "serviceaccount"
                        )
                        or (self.authentication_method == "serviceaccount")
                    )
                )
                and (
                    self.input_method_for_service_account_key
                    and (
                        (
                            hasattr(self.input_method_for_service_account_key, "value")
                            and self.input_method_for_service_account_key.value == "keycontent"
                        )
                        or (self.input_method_for_service_account_key == "keycontent")
                    )
                )
            )
            else exclude.add("service_account_key_content")
        )
        (
            include.add("input_method_for_service_account_key")
            if (
                (
                    (
                        (
                            self.data_source_type
                            and (
                                (
                                    hasattr(self.data_source_type, "value")
                                    and self.data_source_type.value == "Googlebigquery"
                                )
                                or (self.data_source_type == "Googlebigquery")
                            )
                        )
                    )
                )
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "serviceaccount"
                        )
                        or (self.authentication_method == "serviceaccount")
                    )
                )
            )
            else exclude.add("input_method_for_service_account_key")
        )
        (
            include.add("access_token")
            if (
                (
                    (
                        (
                            self.data_source_type
                            and (
                                (
                                    hasattr(self.data_source_type, "value")
                                    and self.data_source_type.value == "Googlebigquery"
                                )
                                or (self.data_source_type == "Googlebigquery")
                            )
                        )
                    )
                )
                and (
                    self.authentication_method
                    and (
                        (hasattr(self.authentication_method, "value") and self.authentication_method.value == "oauth2")
                        or (self.authentication_method == "oauth2")
                    )
                )
            )
            else exclude.add("access_token")
        )
        (
            include.add("refresh_token")
            if (
                (
                    (
                        (
                            self.data_source_type
                            and (
                                (
                                    hasattr(self.data_source_type, "value")
                                    and self.data_source_type.value == "Googlebigquery"
                                )
                                or (self.data_source_type == "Googlebigquery")
                            )
                        )
                    )
                )
                and (
                    self.authentication_method
                    and (
                        (hasattr(self.authentication_method, "value") and self.authentication_method.value == "oauth2")
                        or (self.authentication_method == "oauth2")
                    )
                )
            )
            else exclude.add("refresh_token")
        )
        (
            include.add("keyspace")
            if (
                self.data_source_type
                and (
                    (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                    or (self.data_source_type == "Cassandra")
                )
            )
            else exclude.add("keyspace")
        )
        (
            include.add("cluster_nodes")
            if (
                self.data_source_type
                and (
                    (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                    or (self.data_source_type == "Cassandra")
                )
            )
            else exclude.add("cluster_nodes")
        )
        (
            include.add("delimiter")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                            or (self.data_source_type == "SybaseASE")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
            )
            else exclude.add("delimiter")
        )
        (
            include.add("port")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                            or (self.data_source_type == "Cassandra")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                        or (self.data_source_type == "DB2")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                        or (self.data_source_type == "Oracle")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Teradata")
                        or (self.data_source_type == "Teradata")
                    )
                )
            )
            else exclude.add("port")
        )
        (
            include.add("network_address")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                            or (self.data_source_type == "SybaseASE")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
            )
            else exclude.add("network_address")
        )
        (
            include.add("service_account_email")
            if (
                (
                    (
                        (
                            self.data_source_type
                            and (
                                (
                                    hasattr(self.data_source_type, "value")
                                    and self.data_source_type.value == "Googlebigquery"
                                )
                                or (self.data_source_type == "Googlebigquery")
                            )
                        )
                    )
                )
                and (
                    self.authentication_method
                    and (
                        (
                            hasattr(self.authentication_method, "value")
                            and self.authentication_method.value == "serviceaccount"
                        )
                        or (self.authentication_method == "serviceaccount")
                    )
                )
            )
            else exclude.add("service_account_email")
        )
        (
            include.add("username")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                            or (self.data_source_type == "Cassandra")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                        or (self.data_source_type == "DB2")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                        or (self.data_source_type == "Oracle")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                        or (self.data_source_type == "SybaseASE")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Teradata")
                        or (self.data_source_type == "Teradata")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "use_dsn_name")
                        or (self.data_source_type == "use_dsn_name")
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("database")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                            or (self.data_source_type == "DB2")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                        or (self.data_source_type == "SybaseASE")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Text")
                        or (self.data_source_type == "Text")
                    )
                )
            )
            else exclude.add("database")
        )
        (
            include.add("password")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                            or (self.data_source_type == "Cassandra")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                        or (self.data_source_type == "DB2")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                        or (self.data_source_type == "Oracle")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                        or (self.data_source_type == "SybaseASE")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "use_dsn_name")
                        or (self.data_source_type == "use_dsn_name")
                    )
                )
            )
            else exclude.add("password")
        )
        (
            include.add("hostname_or_ip_address")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                            or (self.data_source_type == "Cassandra")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                        or (self.data_source_type == "DB2")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                        or (self.data_source_type == "Oracle")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
            )
            else exclude.add("hostname_or_ip_address")
        )
        (
            include.add("port")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                            or (self.data_source_type == "Cassandra")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                        or (self.data_source_type == "DB2")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                        or (self.data_source_type == "Oracle")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
            )
            else exclude.add("port")
        )
        (
            include.add("username")
            if (
                (
                    (
                        self.data_source_type
                        and (
                            (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Cassandra")
                            or (self.data_source_type == "Cassandra")
                        )
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2")
                        or (self.data_source_type == "DB2")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2AS400")
                        or (self.data_source_type == "DB2AS400")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "DB2zOS")
                        or (self.data_source_type == "DB2zOS")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "GreenPlum")
                        or (self.data_source_type == "GreenPlum")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Hive")
                        or (self.data_source_type == "Hive")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Impala")
                        or (self.data_source_type == "Impala")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Informix")
                        or (self.data_source_type == "Informix")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (
                            hasattr(self.data_source_type, "value")
                            and self.data_source_type.value == "MicrosoftSQLServer"
                        )
                        or (self.data_source_type == "MicrosoftSQLServer")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MongoDB")
                        or (self.data_source_type == "MongoDB")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "MySQL")
                        or (self.data_source_type == "MySQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Netezza")
                        or (self.data_source_type == "Netezza")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "Oracle")
                        or (self.data_source_type == "Oracle")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "PostgreSQL")
                        or (self.data_source_type == "PostgreSQL")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseASE")
                        or (self.data_source_type == "SybaseASE")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "SybaseIQ")
                        or (self.data_source_type == "SybaseIQ")
                    )
                )
                or (
                    self.data_source_type
                    and (
                        (hasattr(self.data_source_type, "value") and self.data_source_type.value == "use_dsn_name")
                        or (self.data_source_type == "use_dsn_name")
                    )
                )
            )
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")
        (
            include.add("service_account_private_key_file_path")
            if (self.hidden_dummy_property1)
            else exclude.add("service_account_private_key_file_path")
        )

        (
            include.add("keyspace")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Cassandra" in str(self.data_source_type.value)
                    )
                    or ("Cassandra" in str(self.data_source_type))
                )
            )
            else exclude.add("keyspace")
        )
        (
            include.add("authentication_method")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            else exclude.add("authentication_method")
        )
        (
            include.add("service_account_key_content")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "serviceaccount" in str(self.authentication_method.value)
                    )
                    or ("serviceaccount" in str(self.authentication_method))
                )
            )
            and (
                self.input_method_for_service_account_key
                and (
                    (
                        hasattr(self.input_method_for_service_account_key, "value")
                        and self.input_method_for_service_account_key.value
                        and "keycontent" in str(self.input_method_for_service_account_key.value)
                    )
                    or ("keycontent" in str(self.input_method_for_service_account_key))
                )
            )
            else exclude.add("service_account_key_content")
        )
        (
            include.add("network_address")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseASE" in str(self.data_source_type.value)
                    )
                    or ("SybaseASE" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseIQ" in str(self.data_source_type.value)
                    )
                    or ("SybaseIQ" in str(self.data_source_type))
                )
            )
            else exclude.add("network_address")
        )
        (
            include.add("service_account_email")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "serviceaccount" in str(self.authentication_method.value)
                    )
                    or ("serviceaccount" in str(self.authentication_method))
                )
            )
            else exclude.add("service_account_email")
        )
        (
            include.add("use_cas_lite_service")
            if (self.cas_lite_service_authorization_header)
            else exclude.add("use_cas_lite_service")
        )
        (
            include.add("cas_lite_service_authorization_header")
            if (self.use_cas_lite_service == "true" or self.use_cas_lite_service)
            else exclude.add("cas_lite_service_authorization_header")
        )
        (
            include.add("delimiter")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseASE" in str(self.data_source_type.value)
                    )
                    or ("SybaseASE" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseIQ" in str(self.data_source_type.value)
                    )
                    or ("SybaseIQ" in str(self.data_source_type))
                )
            )
            else exclude.add("delimiter")
        )
        (
            include.add("service_account_private_key_file_path")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "serviceaccount" in str(self.authentication_method.value)
                    )
                    or ("serviceaccount" in str(self.authentication_method))
                )
            )
            and (
                self.input_method_for_service_account_key
                and (
                    (
                        hasattr(self.input_method_for_service_account_key, "value")
                        and self.input_method_for_service_account_key.value
                        and "keyfile" in str(self.input_method_for_service_account_key.value)
                    )
                    or ("keyfile" in str(self.input_method_for_service_account_key))
                )
            )
            else exclude.add("service_account_private_key_file_path")
        )
        (
            include.add("input_method_for_service_account_key")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "serviceaccount" in str(self.authentication_method.value)
                    )
                    or ("serviceaccount" in str(self.authentication_method))
                )
            )
            else exclude.add("input_method_for_service_account_key")
        )
        (
            include.add("database")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2" in str(self.data_source_type.value)
                    )
                    or ("DB2" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2AS400" in str(self.data_source_type.value)
                    )
                    or ("DB2AS400" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2zOS" in str(self.data_source_type.value)
                    )
                    or ("DB2zOS" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "GreenPlum" in str(self.data_source_type.value)
                    )
                    or ("GreenPlum" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Hive" in str(self.data_source_type.value)
                    )
                    or ("Hive" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Impala" in str(self.data_source_type.value)
                    )
                    or ("Impala" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Informix" in str(self.data_source_type.value)
                    )
                    or ("Informix" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MicrosoftSQLServer" in str(self.data_source_type.value)
                    )
                    or ("MicrosoftSQLServer" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MongoDB" in str(self.data_source_type.value)
                    )
                    or ("MongoDB" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MySQL" in str(self.data_source_type.value)
                    )
                    or ("MySQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Netezza" in str(self.data_source_type.value)
                    )
                    or ("Netezza" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "PostgreSQL" in str(self.data_source_type.value)
                    )
                    or ("PostgreSQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseASE" in str(self.data_source_type.value)
                    )
                    or ("SybaseASE" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseIQ" in str(self.data_source_type.value)
                    )
                    or ("SybaseIQ" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Teradata" in str(self.data_source_type.value)
                    )
                    or ("Teradata" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Text" in str(self.data_source_type.value)
                    )
                    or ("Text" in str(self.data_source_type))
                )
            )
            else exclude.add("database")
        )
        (
            include.add("cluster_nodes")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Cassandra" in str(self.data_source_type.value)
                    )
                    or ("Cassandra" in str(self.data_source_type))
                )
            )
            else exclude.add("cluster_nodes")
        )
        (
            include.add("hostname_or_ip_address")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Cassandra" in str(self.data_source_type.value)
                    )
                    or ("Cassandra" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2" in str(self.data_source_type.value)
                    )
                    or ("DB2" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2AS400" in str(self.data_source_type.value)
                    )
                    or ("DB2AS400" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2zOS" in str(self.data_source_type.value)
                    )
                    or ("DB2zOS" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "GreenPlum" in str(self.data_source_type.value)
                    )
                    or ("GreenPlum" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Hive" in str(self.data_source_type.value)
                    )
                    or ("Hive" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Impala" in str(self.data_source_type.value)
                    )
                    or ("Impala" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Informix" in str(self.data_source_type.value)
                    )
                    or ("Informix" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MicrosoftSQLServer" in str(self.data_source_type.value)
                    )
                    or ("MicrosoftSQLServer" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MongoDB" in str(self.data_source_type.value)
                    )
                    or ("MongoDB" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MySQL" in str(self.data_source_type.value)
                    )
                    or ("MySQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Netezza" in str(self.data_source_type.value)
                    )
                    or ("Netezza" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Oracle" in str(self.data_source_type.value)
                    )
                    or ("Oracle" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "PostgreSQL" in str(self.data_source_type.value)
                    )
                    or ("PostgreSQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Teradata" in str(self.data_source_type.value)
                    )
                    or ("Teradata" in str(self.data_source_type))
                )
            )
            else exclude.add("hostname_or_ip_address")
        )
        (
            include.add("client_id")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "oauth2" in str(self.authentication_method.value)
                    )
                    or ("oauth2" in str(self.authentication_method))
                )
            )
            else exclude.add("client_id")
        )
        (
            include.add("default_port")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseASE" in str(self.data_source_type.value)
                    )
                    or ("SybaseASE" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseIQ" in str(self.data_source_type.value)
                    )
                    or ("SybaseIQ" in str(self.data_source_type))
                )
            )
            else exclude.add("default_port")
        )
        (
            include.add("password")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Cassandra" in str(self.data_source_type.value)
                    )
                    or ("Cassandra" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2" in str(self.data_source_type.value)
                    )
                    or ("DB2" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2AS400" in str(self.data_source_type.value)
                    )
                    or ("DB2AS400" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2zOS" in str(self.data_source_type.value)
                    )
                    or ("DB2zOS" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "GreenPlum" in str(self.data_source_type.value)
                    )
                    or ("GreenPlum" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Hive" in str(self.data_source_type.value)
                    )
                    or ("Hive" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Impala" in str(self.data_source_type.value)
                    )
                    or ("Impala" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Informix" in str(self.data_source_type.value)
                    )
                    or ("Informix" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MicrosoftSQLServer" in str(self.data_source_type.value)
                    )
                    or ("MicrosoftSQLServer" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MongoDB" in str(self.data_source_type.value)
                    )
                    or ("MongoDB" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MySQL" in str(self.data_source_type.value)
                    )
                    or ("MySQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Netezza" in str(self.data_source_type.value)
                    )
                    or ("Netezza" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Oracle" in str(self.data_source_type.value)
                    )
                    or ("Oracle" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "PostgreSQL" in str(self.data_source_type.value)
                    )
                    or ("PostgreSQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseASE" in str(self.data_source_type.value)
                    )
                    or ("SybaseASE" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseIQ" in str(self.data_source_type.value)
                    )
                    or ("SybaseIQ" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Teradata" in str(self.data_source_type.value)
                    )
                    or ("Teradata" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "use_dsn_name" in str(self.data_source_type.value)
                    )
                    or ("use_dsn_name" in str(self.data_source_type))
                )
            )
            else exclude.add("password")
        )
        (
            include.add("username")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Cassandra" in str(self.data_source_type.value)
                    )
                    or ("Cassandra" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2" in str(self.data_source_type.value)
                    )
                    or ("DB2" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2AS400" in str(self.data_source_type.value)
                    )
                    or ("DB2AS400" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2zOS" in str(self.data_source_type.value)
                    )
                    or ("DB2zOS" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "GreenPlum" in str(self.data_source_type.value)
                    )
                    or ("GreenPlum" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Hive" in str(self.data_source_type.value)
                    )
                    or ("Hive" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Impala" in str(self.data_source_type.value)
                    )
                    or ("Impala" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Informix" in str(self.data_source_type.value)
                    )
                    or ("Informix" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MicrosoftSQLServer" in str(self.data_source_type.value)
                    )
                    or ("MicrosoftSQLServer" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MongoDB" in str(self.data_source_type.value)
                    )
                    or ("MongoDB" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MySQL" in str(self.data_source_type.value)
                    )
                    or ("MySQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Netezza" in str(self.data_source_type.value)
                    )
                    or ("Netezza" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Oracle" in str(self.data_source_type.value)
                    )
                    or ("Oracle" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "PostgreSQL" in str(self.data_source_type.value)
                    )
                    or ("PostgreSQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseASE" in str(self.data_source_type.value)
                    )
                    or ("SybaseASE" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "SybaseIQ" in str(self.data_source_type.value)
                    )
                    or ("SybaseIQ" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Teradata" in str(self.data_source_type.value)
                    )
                    or ("Teradata" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "use_dsn_name" in str(self.data_source_type.value)
                    )
                    or ("use_dsn_name" in str(self.data_source_type))
                )
            )
            else exclude.add("username")
        )
        (
            include.add("client_secret")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "oauth2" in str(self.authentication_method.value)
                    )
                    or ("oauth2" in str(self.authentication_method))
                )
            )
            else exclude.add("client_secret")
        )
        (
            include.add("project")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            else exclude.add("project")
        )
        (
            include.add("log_on_mech")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Teradata" in str(self.data_source_type.value)
                    )
                    or ("Teradata" in str(self.data_source_type))
                )
            )
            else exclude.add("log_on_mech")
        )
        (
            include.add("service_name")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Oracle" in str(self.data_source_type.value)
                    )
                    or ("Oracle" in str(self.data_source_type))
                )
            )
            else exclude.add("service_name")
        )
        (
            include.add("refresh_token")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "oauth2" in str(self.authentication_method.value)
                    )
                    or ("oauth2" in str(self.authentication_method))
                )
            )
            else exclude.add("refresh_token")
        )
        (
            include.add("access_token")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            and (
                self.authentication_method
                and (
                    (
                        hasattr(self.authentication_method, "value")
                        and self.authentication_method.value
                        and "oauth2" in str(self.authentication_method.value)
                    )
                    or ("oauth2" in str(self.authentication_method))
                )
            )
            else exclude.add("access_token")
        )
        (
            include.add("dataset")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Googlebigquery" in str(self.data_source_type.value)
                    )
                    or ("Googlebigquery" in str(self.data_source_type))
                )
            )
            else exclude.add("dataset")
        )
        (
            include.add("port")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Cassandra" in str(self.data_source_type.value)
                    )
                    or ("Cassandra" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2" in str(self.data_source_type.value)
                    )
                    or ("DB2" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2AS400" in str(self.data_source_type.value)
                    )
                    or ("DB2AS400" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "DB2zOS" in str(self.data_source_type.value)
                    )
                    or ("DB2zOS" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "GreenPlum" in str(self.data_source_type.value)
                    )
                    or ("GreenPlum" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Hive" in str(self.data_source_type.value)
                    )
                    or ("Hive" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Impala" in str(self.data_source_type.value)
                    )
                    or ("Impala" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Informix" in str(self.data_source_type.value)
                    )
                    or ("Informix" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MicrosoftSQLServer" in str(self.data_source_type.value)
                    )
                    or ("MicrosoftSQLServer" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MongoDB" in str(self.data_source_type.value)
                    )
                    or ("MongoDB" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "MySQL" in str(self.data_source_type.value)
                    )
                    or ("MySQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Netezza" in str(self.data_source_type.value)
                    )
                    or ("Netezza" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Oracle" in str(self.data_source_type.value)
                    )
                    or ("Oracle" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "PostgreSQL" in str(self.data_source_type.value)
                    )
                    or ("PostgreSQL" in str(self.data_source_type))
                )
                and self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "Teradata" in str(self.data_source_type.value)
                    )
                    or ("Teradata" in str(self.data_source_type))
                )
            )
            else exclude.add("port")
        )
        (
            include.add("data_source_name")
            if (
                self.data_source_type
                and (
                    (
                        hasattr(self.data_source_type, "value")
                        and self.data_source_type.value
                        and "use_dsn_name" in str(self.data_source_type.value)
                    )
                    or ("use_dsn_name" in str(self.data_source_type))
                )
            )
            else exclude.add("data_source_name")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



import threading
import time


class CooldownLock:
    def __init__(self, cooldown: float):
        self._lock = threading.Lock()
        self._cooldown = cooldown
        self._earliest_use_time = 0

    def acquire(self):
        self._lock.acquire()

        remaining_cooldown = self._earliest_use_time - time.monotonic()
        if remaining_cooldown > 0:
            time.sleep(remaining_cooldown)

    def release(self):
        if not self._lock.locked():
            raise RuntimeError("Lock not held")

        self._earliest_use_time = time.monotonic() + self._cooldown
        self._lock.release()

    def __enter__(self) -> "CooldownLock":
        self.acquire()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.release()



from ibm_watsonx_data_integration.services.datastage.models.connections.base_connection import BaseConnection
from pydantic import ConfigDict, Field
from typing import ClassVar


class BoxConn(BaseConnection):
    model_config = ConfigDict(populate_by_name=True)

    datasource_type: ClassVar[str] = "99c3c67b-2133-4006-81f6-2b375a0048a3"
    asset_id: str | None = None
    proj_id: str | None = None
    name: str = Field(None, alias="name")
    ds_host: str | None = Field(None, alias="_host")
    ds_port: str | None = Field(None, alias="_port")
    access_token: str | None = Field(None, alias="access_token")
    client_id: str = Field(None, alias="client_id")
    client_secret: str = Field(None, alias="client_secret")
    cluster_access_token: str | None = Field(None, alias="cluster_access_token")
    cluster_user_name: str | None = Field(None, alias="cluster_user_name")
    defer_credentials: bool | None = Field(False, alias="defer_credentials")
    enterprise_id: str = Field(None, alias="enterprise_id")
    private_key: str = Field(None, alias="private_key")
    private_key_password: str | None = Field(None, alias="private_key_password")
    public_key: str = Field(None, alias="public_key")
    username: str | None = Field(None, alias="username")
    vaulted_properties: str | None = Field(None, alias="vaulted_properties")
    hidden_dummy_property1: str | None = Field(None, alias="hiddenDummyProperty1")
    hidden_dummy_property2: str | None = Field(None, alias="hiddenDummyProperty2")

    def validate(self):
        include = set()
        exclude = set()

        include.add("access_token") if (not self.defer_credentials) else exclude.add("access_token")
        include.add("public_key") if (not self.defer_credentials) else exclude.add("public_key")
        (
            include.add("private_key_password")
            if (((not self.defer_credentials)) and (not self.access_token))
            else exclude.add("private_key_password")
        )
        (
            include.add("private_key")
            if (((not self.defer_credentials)) and (not self.access_token))
            else exclude.add("private_key")
        )
        (
            include.add("client_secret")
            if (((not self.defer_credentials)) and (not self.access_token))
            else exclude.add("client_secret")
        )
        (
            include.add("enterprise_id")
            if (((not self.defer_credentials)) and (not self.access_token))
            else exclude.add("enterprise_id")
        )
        (
            include.add("client_id")
            if (((not self.defer_credentials)) and (not self.access_token))
            else exclude.add("client_id")
        )
        (
            include.add("username")
            if (((not self.defer_credentials)) and (not self.access_token))
            else exclude.add("username")
        )
        (
            include.add("hidden_dummy_property2")
            if (self.hidden_dummy_property1)
            else exclude.add("hidden_dummy_property2")
        )
        (
            include.add("hidden_dummy_property1")
            if (self.hidden_dummy_property2)
            else exclude.add("hidden_dummy_property1")
        )
        include.add("vaulted_properties") if (self.hidden_dummy_property1) else exclude.add("vaulted_properties")

        (
            include.add("enterprise_id")
            if (not self.defer_credentials) and (not self.access_token)
            else exclude.add("enterprise_id")
        )
        include.add("public_key") if (not self.defer_credentials) else exclude.add("public_key")
        (
            include.add("client_id")
            if (not self.defer_credentials) and (not self.access_token)
            else exclude.add("client_id")
        )
        (
            include.add("private_key_password")
            if (not self.defer_credentials) and (not self.access_token)
            else exclude.add("private_key_password")
        )
        include.add("access_token") if (not self.defer_credentials) else exclude.add("access_token")
        (
            include.add("private_key")
            if (not self.defer_credentials) and (not self.access_token)
            else exclude.add("private_key")
        )
        include.add("username") if (not self.defer_credentials) and (not self.access_token) else exclude.add("username")
        (
            include.add("client_secret")
            if (not self.defer_credentials) and (not self.access_token)
            else exclude.add("client_secret")
        )
        return include, exclude

    @classmethod
    def from_dict(cls, properties):
        return cls.model_construct(**properties)



import concurrent.futures
import json
import multiprocessing
import os
import re
import shutil
import subprocess
import time
from ibm_watsonx_data_integration.services.datastage.codegen.python_generator import *

global lock
lock = multiprocessing.Lock()


def _sort_keys(d):
    if not isinstance(d, dict):
        return d
    return {k: _sort_keys(d[k]) for k in sorted(d.keys(), key=lambda x: str(x))}


###############################################################################


def _filter_ignored(errors):
    errors_filtered = dict()

    for p, err in errors.items():
        if "SuperNodes are not supported" in err:
            continue  # Ignore subflows
        elif bool(re.search(r"ValueError: .*? is not a valid .*?", err)):
            continue  # Ignore parameter/value sets
        else:
            errors_filtered[p] = err

    return errors_filtered


# Internal helper that runs PythonGenerator on one zip
def _generate_for_one_zip(i, filename, input_dir, output_dir, generator: PythonGenerator, errors):
    if filename.endswith(".zip"):
        folder = filename.replace(".zip", "")
        print(f"({i}) Regenerating folder {folder}")
        input_path = os.path.join(input_dir, filename)
        output_path = os.path.join(output_dir, folder)
        _, errors_for_zip = generator.generate(input_path=input_path, output_path=output_path)

        # Filter out errors that aren't important
        errors_filtered = _filter_ignored(errors_for_zip)
        if errors_filtered:
            errors[folder] = errors_filtered

    return errors


# Internal helper that runs PythonGenerator on one json
def _generate_for_one_json(i, filename, input_dir, output_dir, generator: PythonGenerator, errors):
    if filename.endswith(".json"):
        input_path = os.path.join(input_dir, filename)
        output_path = os.path.join(output_dir, filename.replace(".json", ".py"))
        _, errors_for_json = generator.generate(input_path=input_path, output_path=output_path)

        # Filter out errors that aren't important
        errors_filtered = _filter_ignored(errors_for_json)
        if errors_filtered:
            errors[filename.replace(".json", ".py")] = errors_filtered

    return errors


# Run PythonGenerator on all files in the input_dir
def generate(file_type, input_dir, output_dir, results_dir, generator):
    os.makedirs(output_dir, exist_ok=True)
    os.makedirs(results_dir, exist_ok=True)
    helper = _generate_for_one_zip if file_type == "zip" else _generate_for_one_json

    start_time = time.time()

    manager = multiprocessing.Manager()
    errors = manager.dict()
    with concurrent.futures.ProcessPoolExecutor(max_workers=12) as executor:
        filenames = [filename for filename in os.listdir(input_dir)]
        futures = [
            executor.submit(helper, i, filename, input_dir, output_dir, generator, errors)
            for i, filename in enumerate(filenames)
        ]
        for future in futures:
            future.result()

    end_time = time.time()
    elapsed_time = end_time - start_time
    print(f"Elapsed time: {elapsed_time} seconds")

    results_path = os.path.join(results_dir, f"generate_errors_{file_type}.json")
    with open(results_path, "w") as f:
        json.dump(_sort_keys(dict(errors)), f, indent=4)
    return _sort_keys(dict(errors))


###############################################################################


# Helper to execute one code file at 'code_path' and return the prints/errors
def _execute_code(id, code_path, temp_dir, prints, errors, successes):
    global lock
    os.makedirs(temp_dir, exist_ok=True)

    # Modifications: replace placeholder api key and project id, then save temporarily
    with open(code_path) as f:
        original = f.read()
    modified = original.replace(
        "# This code was autogenerated by PythonGenerator",
        "# This code was autogenerated by PythonGenerator\nimport os",
    )
    modified = modified.replace('api_key = "<your_api_key>"', 'api_key = os.environ["WXDI_API_KEY"]')
    modified = modified.replace('guid="<your_project_id>"', 'guid=os.environ["WXDI_PROJECT_ID"]')
    file_name = f"{id}_{Path(code_path).stem}"
    temp_code_path = os.path.join(temp_dir, f"temp_{file_name}.py")
    os.makedirs(os.path.dirname(temp_code_path), exist_ok=True)
    with open(temp_code_path, "w") as temp_file:
        temp_file.write(modified)

    # Run it and return any print outputs and errors
    timeout = 50
    result_print, result_error = [], []
    try:
        result = subprocess.run(["python", temp_code_path], capture_output=True, text=True, timeout=timeout)
        os.remove(temp_code_path)
        result_print = result.stdout
        result_error = result.stderr
    except subprocess.TimeoutExpired:
        result_print = ""
        result_error = "timed out"

    # Process results
    with lock:
        if not result_print and not result_error:
            successes.append(file_name)
            print(id, "success")
        if result_print:
            prints[file_name] = result_print
            print(id, "print")
        if result_error:
            errors[file_name] = result_error
            print(id, "error")


# Saves a checkpoint of prints, errors, and successes
def save_checkpoint(i, prints, errors, successes, result_dir):
    print(f"Saving checkpoint {i}: {len(prints)} Print | {len(errors)} Error | {len(successes)} Success")

    with open(os.path.join(result_dir, "run_print.json"), "w") as f:
        json.dump(_sort_keys(dict(prints)), f, indent=4)
    with open(os.path.join(result_dir, "run_error.json"), "w") as f:
        json.dump(_sort_keys(dict(errors)), f, indent=4)
    with open(os.path.join(result_dir, "run_success.json"), "w") as f:
        json.dump(sorted(successes), f, indent=4)


# Run generated code
def execute_code(code_dir, results_dir, temp_dir):
    manager = multiprocessing.Manager()
    errors = manager.dict()
    prints = manager.dict()
    successes = manager.list()

    # Collect valid code files to run
    valid_files = []
    for root, _, files in os.walk(code_dir):
        for file in files:
            full_path = os.path.join(root, file)
            if full_path.endswith(".py"):
                with open(full_path, encoding="utf-8") as f:
                    first_line = f.readline().strip()
                    if first_line == "# This code was autogenerated by PythonGenerator":
                        valid_files.append(full_path)

    # Run code files in parallel
    start_time = time.time()
    with concurrent.futures.ProcessPoolExecutor(max_workers=12) as executor:
        futures = []
        for i, code_path in enumerate(valid_files):
            futures.append(executor.submit(_execute_code, i, code_path, temp_dir, prints, errors, successes))
            # Checkpoint
            if (i + 1) % 50 == 0:
                concurrent.futures.wait(futures)
                save_checkpoint(i, prints, errors, successes, results_dir)
                futures = []

        # Final Checkpoint
        concurrent.futures.wait(futures)
        save_checkpoint(i, prints, errors, successes, results_dir)

    end_time = time.time()
    elapsed_time = end_time - start_time
    print(f"Elapsed time: {elapsed_time} seconds")


###############################################################################

if __name__ == "__main__":
    test_dir = os.path.join(os.getcwd(), "ibm_watsonx_data_integration/services/datastage/tests/pythongenerator")
    results_dir = os.path.join(test_dir, "results")
    temp_dir = os.path.join(test_dir, "temp")
    os.makedirs(temp_dir, exist_ok=True)

    # Choose PythonGenerator settings here - TODO: need to test both modes next
    generator = PythonGenerator()
    generator.configuration.mode = "file_per_flow"  # or single_file
    generator.configuration.api_key = 'os.environ["WXDI_API_KEY"]'
    generator.configuration.project_id = 'os.environ["WXDI_PROJECT_ID"]'
    generator.configuration.create_job = False
    generator.configuration.run_job = False

    # Run PythonGenerator on zip format
    # zip_dir = os.path.join(test_dir, "flows/zip")
    # code_dir = os.path.join(test_dir, "code/zip")
    # generate("zip", zip_dir, code_dir, results_dir, generator)

    # Run PythonGenerator on one zip - debug only
    # zip_name = "001"
    # shutil.copy2(os.path.join(test_dir, f"flows/zip/{zip_name}.zip"), temp_dir)
    # generate("zip", temp_dir, os.path.join(test_dir, "code/zip"), results_dir, generator)
    # os.remove(os.path.join(temp_dir, zip_name + ".zip"))

    # Run conversion for connector tests - zip
    shutil.copy2(os.path.join(test_dir, "flows/zip/connector_tests.zip"), temp_dir)
    generate("zip", temp_dir, os.path.join(test_dir, "code/zip"), results_dir, generator)
    os.remove(os.path.join(temp_dir, "connector_tests.zip"))

    # Run conversion for connector tests - json
    # input_dir = os.path.join(test_dir, "flows/json/connector")
    # output_dir = os.path.join(test_dir, "code/json/connector")
    # generate("json", input_dir, output_dir, results_dir, generator)

    # Run PythonGnenerator on json format of customer zips
    # all_errors = dict()
    # for i in range(1, 562):
    #     json_dir = os.path.join(test_dir, "flows/json", f"{i:03}")
    #     code_dir = os.path.join(test_dir, "code/json", f"{i:03}")
    #     curr_errors = generate("json", json_dir, code_dir, results_dir, generator)
    #     if curr_errors: all_errors[f"{i:03}"] = curr_errors
    #     if i > 0 and i % 20 == 0:
    #         print(f"Saving checkpoint {i} with {len(curr_errors.keys())} folders with errors")
    #         with open(os.path.join(results_dir, "generate_errors_json.json"), "w") as f:
    #             json.dump(all_errors, f, indent=4)

    # Run generated code - make sure to set WXDI_API_KEY and WXDI_PROJECT_ID
    # code_dir = os.path.join(test_dir, "code/custom")
    # execute_code(code_dir, results_dir, temp_dir)



import warnings
from ibm_watsonx_data_integration.services.datastage.models.base_stage import BaseStage
from ibm_watsonx_data_integration.services.datastage.models.enums import COLUMN_IMPORT
from pydantic import Field
from typing import ClassVar


class column_import(BaseStage):

    op_name: ClassVar[str] = "PxColumnImport"
    node_type: ClassVar[str] = "execution_node"
    image: ClassVar[str] = "/data-intg/flows/graphics/palette/PxColumnImport.svg"
    label: ClassVar[str] = "Column Import"
    runtime_column_propagation: bool = Field(False, alias="runtime_column_propagation")
    actual_length: int | None = Field(None, alias="actual_length")
    allow_all_zeros: COLUMN_IMPORT.AllowAllZeros | None = Field(
        COLUMN_IMPORT.AllowAllZeros.nofix_zero, alias="allow_all_zeros"
    )
    allow_signed_import: COLUMN_IMPORT.AllowSignedImport | None = Field(
        COLUMN_IMPORT.AllowSignedImport.allow_signed_import, alias="allow_signed_import"
    )
    auto_column_propagation: bool | None = Field(None, alias="auto_column_propagation")
    buf_free_run: int | None = Field(50, alias="buf_free_run")
    buf_free_run_ronly: int | None = Field(50, alias="buf_free_run_ronly")
    buf_mode: COLUMN_IMPORT.BufMode | None = Field(COLUMN_IMPORT.BufMode.default, alias="buf_mode")
    buf_mode_ronly: COLUMN_IMPORT.BufModeRonly | None = Field(
        COLUMN_IMPORT.BufModeRonly.default, alias="buf_mode_ronly"
    )
    byte_order: COLUMN_IMPORT.ByteOrder | None = Field(COLUMN_IMPORT.ByteOrder.native_endian, alias="byte_order")
    c_format: str | None = Field("", alias="c_format")
    charset: COLUMN_IMPORT.Charset | None = Field(COLUMN_IMPORT.Charset.ascii, alias="charset")
    check_intact: COLUMN_IMPORT.CheckIntact | None = Field(
        COLUMN_IMPORT.CheckIntact.check_intact, alias="check_intact"
    )
    check_intact_flag: bool | None = Field(False, alias="check_intact_flag")
    coll_type: COLUMN_IMPORT.CollType | None = Field(COLUMN_IMPORT.CollType.auto, alias="coll_type")
    combinability: COLUMN_IMPORT.Combinability | None = Field(
        COLUMN_IMPORT.Combinability.auto, alias="combinability"
    )
    current_output_link_type: str = Field("PRIMARY", alias="currentOutputLinkType")
    data_format: COLUMN_IMPORT.DataFormat | None = Field(COLUMN_IMPORT.DataFormat.text, alias="data_format")
    date_format: str | None = Field("", alias="date_format")
    date_option: COLUMN_IMPORT.DateOption | None = Field(COLUMN_IMPORT.DateOption.none, alias="dateOption")
    days_since: str | None = Field("", alias="days_since")
    decimal_options: COLUMN_IMPORT.DecimalOption | None = Field([], alias="decimalOption")
    decimal_packed: COLUMN_IMPORT.DecimalPacked | None = Field(
        COLUMN_IMPORT.DecimalPacked.packed, alias="decimal_packed"
    )
    decimal_packed_check: COLUMN_IMPORT.DecimalPackedCheck | None = Field(
        COLUMN_IMPORT.DecimalPackedCheck.check, alias="decimal_packed_check"
    )
    decimal_packed_sign_position: COLUMN_IMPORT.DecimalPackedSignPosition | None = Field(
        COLUMN_IMPORT.DecimalPackedSignPosition.trailing, alias="decimal_packed_sign_position"
    )
    decimal_packed_signed: COLUMN_IMPORT.DecimalPackedSigned | None = Field(
        COLUMN_IMPORT.DecimalPackedSigned.signed, alias="decimal_packed_signed"
    )
    decimal_sep_value: str | None = Field("", alias="decimal_sep_value")
    decimal_separator: COLUMN_IMPORT.DecimalSeparator | None = Field(
        COLUMN_IMPORT.DecimalSeparator.period, alias="decimal_separator"
    )
    delim: COLUMN_IMPORT.Delim | None = Field(COLUMN_IMPORT.Delim.comma, alias="delim")
    delim_string: str | None = Field("", alias="delim_string")
    delim_value: str | None = Field("", alias="delim_value")
    disk_write_inc: int | None = Field(1048576, alias="disk_write_inc")
    disk_write_inc_ronly: int | None = Field(1048576, alias="disk_write_inc_ronly")
    enable_flow_acp_control: bool = Field(True, alias="enableFlowAcpControl")
    enable_schemaless_design: bool = Field(False, alias="enableSchemalessDesign")
    execmode: COLUMN_IMPORT.Execmode | None = Field(COLUMN_IMPORT.Execmode.default_par, alias="execmode")
    export_ebcdic_as_ascii: COLUMN_IMPORT.ExportEbcdicAsAscii | None = Field(
        COLUMN_IMPORT.ExportEbcdicAsAscii.export_ebcdic_as_ascii, alias="export_ebcdic_as_ascii"
    )
    field: str = Field(" ", alias="field")
    field_option: COLUMN_IMPORT.FieldOption | None = Field(COLUMN_IMPORT.FieldOption.delimiter, alias="fieldOption")
    fill: COLUMN_IMPORT.Fill | None = Field(COLUMN_IMPORT.Fill.null, alias="fill")
    fill_char_value: str | None = Field("", alias="fill_char_value")
    final_delim: COLUMN_IMPORT.FinalDelim | None = Field(COLUMN_IMPORT.FinalDelim.end, alias="final_delim")
    final_delim_string: str | None = Field("", alias="final_delim_string")
    final_delim_value: str | None = Field("", alias="final_delim_value")
    flow_dirty: str | None = Field("false", alias="flow_dirty")
    general_option: COLUMN_IMPORT.GeneralOption | None = Field([], alias="generalOption")
    hide: bool | None = Field(False, alias="hide")
    import_ascii_as_ebcdic: COLUMN_IMPORT.ImportAsciiAsEbcdic | None = Field(
        COLUMN_IMPORT.ImportAsciiAsEbcdic.import_ascii_as_ebcdic, alias="import_ascii_as_ebcdic"
    )
    in_format: str | None = Field("", alias="in_format")
    input_count: int | None = Field(0, alias="input_count")
    input_link_description: list | None = Field("", alias="inputLinkDescription")
    inputcol_properties: list | None = Field([], alias="inputcolProperties")
    intact: str | None = Field("", alias="intact")
    is_julian: COLUMN_IMPORT.IsJulian | None = Field(COLUMN_IMPORT.IsJulian.julian, alias="is_julian")
    is_midnight_seconds: COLUMN_IMPORT.IsMidnightSeconds | None = Field(
        COLUMN_IMPORT.IsMidnightSeconds.midnight_seconds, alias="is_midnight_seconds"
    )
    keep_field: COLUMN_IMPORT.KeepField | None = Field(COLUMN_IMPORT.KeepField.false, alias="keepField")
    key_col_select: COLUMN_IMPORT.KeyColSelect | None = Field(
        COLUMN_IMPORT.KeyColSelect.default, alias="keyColSelect"
    )
    key_cols_coll: list | None = Field([], alias="keyColsColl")
    key_cols_coll_ordered: list | None = Field([], alias="keyColsCollOrdered")
    key_cols_coll_robin: list | None = Field([], alias="keyColsCollRobin")
    key_cols_none: list | None = Field([], alias="keyColsNone")
    key_cols_part: list | None = Field([], alias="keyColsPart")
    max_mem_buf_size: int | None = Field(3145728, alias="max_mem_buf_size")
    max_mem_buf_size_ronly: int | None = Field(3145728, alias="max_mem_buf_size_ronly")
    max_width: int | None = Field(None, alias="max_width")
    null_field: str | None = Field("NULL", alias="null_field")
    null_field_sep: COLUMN_IMPORT.NullFieldSep | None = Field(
        COLUMN_IMPORT.NullFieldSep.comma, alias="null_field_sep"
    )
    null_field_sep_flag: bool | None = Field(False, alias="null_field_sep_flag")
    null_field_sep_value: str | None = Field("", alias="null_field_sep_value")
    null_length: int | None = Field(None, alias="null_length")
    numeric_option: COLUMN_IMPORT.NumericOption | None = Field([], alias="numericOption")
    out_format: str | None = Field("", alias="out_format")
    output_acp_should_hide: bool = Field(True, alias="outputAcpShouldHide")
    output_count: int | None = Field(0, alias="output_count")
    output_link_description: list | None = Field("", alias="outputLinkDescription")
    outputcol_properties: list | None = Field([], alias="outputcolProperties")
    padchar: COLUMN_IMPORT.Padchar | None = Field(COLUMN_IMPORT.Padchar.false_, alias="padchar")
    padchar_value: str | None = Field("", alias="padchar_value")
    part_client_dbname: str | None = Field(None, alias="part_client_dbname")
    part_client_instance: str | None = Field(None, alias="part_client_instance")
    part_dbconnection: str | None = Field("", alias="part_dbconnection")
    part_stable: bool | None = Field(None, alias="part_stable")
    part_stable_coll: bool | None = Field(False, alias="part_stable_coll")
    part_stable_ordered: bool | None = Field(False, alias="part_stable_ordered")
    part_stable_roundrobin_coll: bool | None = Field(False, alias="part_stable_roundrobin_coll")
    part_table: str | None = Field(None, alias="part_table")
    part_type: COLUMN_IMPORT.PartType | None = Field(COLUMN_IMPORT.PartType.auto, alias="part_type")
    part_unique: bool | None = Field(None, alias="part_unique")
    part_unique_coll: bool | None = Field(False, alias="part_unique_coll")
    part_unique_ordered: bool | None = Field(False, alias="part_unique_ordered")
    part_unique_roundrobin_coll: bool | None = Field(False, alias="part_unique_roundrobin_coll")
    perform_sort: bool | None = Field(False, alias="perform_sort")
    perform_sort_coll: bool | None = Field(False, alias="perform_sort_coll")
    perform_sort_modulus: bool | None = Field(False, alias="perform_sort_modulus")
    precision: int | None = Field(None, alias="precision")
    prefix: COLUMN_IMPORT.Prefix | None = Field(COLUMN_IMPORT.Prefix.one, alias="prefix")
    preserve: COLUMN_IMPORT.Preserve | None = Field(COLUMN_IMPORT.Preserve.default_propagate, alias="preserve")
    print_field: COLUMN_IMPORT.PrintField | None = Field(COLUMN_IMPORT.PrintField.print_field, alias="print_field")
    queue_upper_size: int | None = Field(0, alias="queue_upper_size")
    queue_upper_size_ronly: int | None = Field(0, alias="queue_upper_size_ronly")
    quote: COLUMN_IMPORT.Quote | None = Field(COLUMN_IMPORT.Quote.double, alias="quote")
    quote_value: str | None = Field("", alias="quote_value")
    rec_level_option: COLUMN_IMPORT.RecLevelOption | None = Field([], alias="recLevelOption")
    record_delim: COLUMN_IMPORT.RecordDelim | None = Field(COLUMN_IMPORT.RecordDelim.newline, alias="record_delim")
    record_delim_string: str | None = Field("", alias="record_delim_string")
    record_delim_value: str | None = Field("", alias="record_delim_value")
    record_format: COLUMN_IMPORT.RecordFormat | None = Field(
        COLUMN_IMPORT.RecordFormat.type_implicit, alias="record_format"
    )
    record_len_value: int | None = Field(0, alias="record_len_value")
    record_length: COLUMN_IMPORT.RecordLength | None = Field(COLUMN_IMPORT.RecordLength.fixed, alias="record_length")
    record_prefix: COLUMN_IMPORT.RecordPrefix | None = Field(COLUMN_IMPORT.RecordPrefix.one, alias="record_prefix")
    round: COLUMN_IMPORT.Round | None = Field(COLUMN_IMPORT.Round.trunc_zero, alias="round")
    runtime_column_propagation: bool | None = Field(None, alias="runtime_column_propagation")
    save_rejects_fail_rejects: COLUMN_IMPORT.SaveRejectsFailRejects | None = Field(
        COLUMN_IMPORT.SaveRejectsFailRejects.warn, alias="saveRejects-failRejects"
    )
    scale: int | None = Field(None, alias="scale")
    schema_: list | None = Field([], alias="schema")
    schemafile: str = Field(None, alias="schemafile")
    selection: COLUMN_IMPORT.Selection | None = Field(COLUMN_IMPORT.Selection.explicit, alias="selection")
    show_coll_type: int | None = Field(0, alias="showCollType")
    show_part_type: int | None = Field(1, alias="showPartType")
    show_sort_options: int | None = Field(0, alias="showSortOptions")
    sort_instructions: str | None = Field("", alias="sortInstructions")
    sort_instructions_text: str | None = Field("", alias="sortInstructionsText")
    stage_description: list | None = Field("", alias="stageDescription")
    string_option: COLUMN_IMPORT.StringOption | None = Field([], alias="stringOption")
    time_format: str | None = Field("", alias="time_format")
    time_option: COLUMN_IMPORT.TimeOption | None = Field(COLUMN_IMPORT.TimeOption.none, alias="timeOption")
    timestamp_format: str | None = Field("", alias="timestamp_format")
    timestamp_option: COLUMN_IMPORT.TimestampOption | None = Field(
        COLUMN_IMPORT.TimestampOption.none, alias="timestampOption"
    )
    vector_prefix: COLUMN_IMPORT.VectorPrefix | None = Field(COLUMN_IMPORT.VectorPrefix.one, alias="vector_prefix")
    width: int | None = Field(None, alias="width")

    def validate_parameters(self) -> tuple[set, set]:
        include = set()
        exclude = set()

        (
            include.add("schemafile")
            if (

                    self.selection
                    and (
                        (hasattr(self.selection, "value") and self.selection.value == "file")
                        or (self.selection == "file")
                    )

            )
            else exclude.add("schemafile")
        )
        (
            include.add("schema_")
            if (

                    self.selection
                    and (
                        (hasattr(self.selection, "value") and self.selection.value == "explicit")
                        or (self.selection == "explicit")
                    )

            )
            else exclude.add("schema_")
        )
        include.add("preserve") if (self.output_count and self.output_count > 0) else exclude.add("preserve")
        include.add("max_mem_buf_size") if (self.buf_mode != "nobuffer") else exclude.add("max_mem_buf_size")
        include.add("buf_free_run") if (self.buf_mode != "nobuffer") else exclude.add("buf_free_run")
        include.add("queue_upper_size") if (self.buf_mode != "nobuffer") else exclude.add("queue_upper_size")
        include.add("disk_write_inc") if (self.buf_mode != "nobuffer") else exclude.add("disk_write_inc")

        (
            include.add("runtime_column_propagation")
            if (not self.enable_schemaless_design)
            else exclude.add("runtime_column_propagation")
        )
        (
            include.add("auto_column_propagation")
            if (not self.output_acp_should_hide)
            else exclude.add("auto_column_propagation")
        )
        (
            include.add("max_mem_buf_size_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("max_mem_buf_size_ronly")
        )
        (
            include.add("buf_free_run_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("buf_free_run_ronly")
        )
        (
            include.add("queue_upper_size_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("queue_upper_size_ronly")
        )
        (
            include.add("disk_write_inc_ronly")
            if (self.buf_mode_ronly != "nobuffer")
            else exclude.add("disk_write_inc_ronly")
        )
        (
            include.add("part_stable")
            if (
                ((self.show_part_type))
                and (self.part_type != "auto")
                and (self.part_type != "db2connector")
                and (self.show_sort_options)
            )
            else exclude.add("part_stable")
        )
        (
            include.add("part_unique")
            if (
                ((self.show_part_type))
                and (self.part_type != "auto")
                and (self.part_type != "db2connector")
                and (self.show_sort_options)
            )
            else exclude.add("part_unique")
        )
        (
            include.add("key_cols_part")
            if (
                (
                    ((self.show_part_type))
                    and ((not self.show_coll_type))
                    and ((self.part_type != "auto"))
                    and ((self.part_type != "db2connector"))
                    and ((self.part_type != "modulus"))
                )
                or (
                    ((self.show_part_type))
                    and (not self.show_coll_type)
                    and (self.part_type == "modulus")
                    and (self.perform_sort_modulus)
                )
            )
            else exclude.add("key_cols_part")
        )
        (
            include.add("part_dbconnection")
            if (((self.part_type == "db2part")) and (self.show_part_type))
            else exclude.add("part_dbconnection")
        )
        (
            include.add("part_client_dbname")
            if (((self.part_type == "db2part")) and (self.show_part_type))
            else exclude.add("part_client_dbname")
        )
        (
            include.add("part_client_instance")
            if (((self.part_type == "db2part")) and (self.show_part_type))
            else exclude.add("part_client_instance")
        )
        (
            include.add("part_table")
            if (((self.part_type == "db2part")) and (self.show_part_type))
            else exclude.add("part_table")
        )
        (
            include.add("perform_sort")
            if (((self.show_part_type)) and (((self.part_type == "hash")) or (self.part_type == "range")))
            else exclude.add("perform_sort")
        )
        (
            include.add("perform_sort_modulus")
            if (((self.show_part_type)) and (self.part_type == "modulus"))
            else exclude.add("perform_sort_modulus")
        )
        (
            include.add("key_col_select")
            if (
                ((self.show_part_type))
                and (self.part_type == "modulus")
                and (not self.perform_sort_modulus)
            )
            else exclude.add("key_col_select")
        )
        (
            include.add("sort_instructions")
            if (
                ((self.show_part_type))
                and (
                    ((self.part_type == "db2part"))
                    or (self.part_type == "entire")
                    or (self.part_type == "random")
                    or (self.part_type == "roundrobin")
                    or (self.part_type == "same")
                )
            )
            else exclude.add("sort_instructions")
        )
        (
            include.add("sort_instructions_text")
            if (
                ((self.show_part_type))
                and (
                    ((self.part_type == "db2part"))
                    or (self.part_type == "entire")
                    or (self.part_type == "random")
                    or (self.part_type == "roundrobin")
                    or (self.part_type == "same")
                )
            )
            else exclude.add("sort_instructions_text")
        )
        include.add("coll_type") if (self.show_coll_type) else exclude.add("coll_type")
        include.add("part_type") if (self.show_part_type) else exclude.add("part_type")
        (
            include.add("perform_sort_coll")
            if (
                (
                    ((self.show_coll_type))
                    and (
                        ((self.coll_type == "ordered"))
                        or ((self.coll_type == "roundrobin_coll"))
                        or ((self.coll_type == "sortmerge"))
                    )
                )
                or (((not self.show_part_type)) and (not self.show_coll_type))
            )
            else exclude.add("perform_sort_coll")
        )
        (
            include.add("key_cols_coll")
            if (
                ((self.show_coll_type))
                and (not self.show_part_type)
                and (self.coll_type != "auto")
                and (((self.coll_type == "sortmerge")) or (self.perform_sort_coll))
            )
            else exclude.add("key_cols_coll")
        )
        (
            include.add("key_cols_none")
            if (
                ((not self.show_part_type))
                and (not self.show_coll_type)
                and (self.perform_sort_coll)
            )
            else exclude.add("key_cols_none")
        )
        (
            include.add("part_stable_coll")
            if (
                (((self.perform_sort_coll)))
                and (
                    (
                        ((not self.show_part_type))
                        and ((self.show_coll_type))
                        and ((self.coll_type != "auto"))
                        and ((self.show_sort_options))
                    )
                    or (
                        ((not self.show_part_type))
                        and (not self.show_coll_type)
                        and (self.show_sort_options)
                    )
                )
            )
            else exclude.add("part_stable_coll")
        )
        (
            include.add("part_unique_coll")
            if (
                (((self.perform_sort_coll)))
                and (
                    (
                        ((not self.show_part_type))
                        and ((self.show_coll_type))
                        and ((self.coll_type != "auto"))
                        and ((self.show_sort_options))
                    )
                    or (
                        ((not self.show_part_type))
                        and (not self.show_coll_type)
                        and (self.show_sort_options)
                    )
                )
            )
            else exclude.add("part_unique_coll")
        )
        (
            include.add("final_delim")
            if (self.rec_level_option and "final_delimiter" in str(self.rec_level_option))
            else exclude.add("final_delim")
        )
        (
            include.add("final_delim_value")
            if (
                ((self.rec_level_option and "final_delimiter" in str(self.rec_level_option)))
                and (self.final_delim == " ")
            )
            else exclude.add("final_delim_value")
        )
        (
            include.add("fill")
            if (self.rec_level_option and "fill" in str(self.rec_level_option))
            else exclude.add("fill")
        )
        (
            include.add("fill_char_value")
            if (((self.rec_level_option and "fill" in str(self.rec_level_option))) and (self.fill == -1))
            else exclude.add("fill_char_value")
        )
        (
            include.add("final_delim_string")
            if (self.rec_level_option and "final_delim_string" in str(self.rec_level_option))
            else exclude.add("final_delim_string")
        )
        (
            include.add("intact")
            if (self.rec_level_option and "intact" in str(self.rec_level_option))
            else exclude.add("intact")
        )
        (
            include.add("check_intact_flag")
            if (self.rec_level_option and "intact" in str(self.rec_level_option))
            else exclude.add("check_intact_flag")
        )
        (
            include.add("check_intact")
            if (
                ((self.check_intact_flag))
                and (self.rec_level_option and "intact" in str(self.rec_level_option))
            )
            else exclude.add("check_intact")
        )
        (
            include.add("record_delim")
            if (self.rec_level_option and "record_delimiter" in str(self.rec_level_option))
            else exclude.add("record_delim")
        )
        (
            include.add("record_delim_value")
            if (
                ((self.rec_level_option and "record_delimiter" in str(self.rec_level_option)))
                and (self.record_delim == " ")
            )
            else exclude.add("record_delim_value")
        )
        (
            include.add("record_delim_string")
            if (self.rec_level_option and "record_delim_string" in str(self.rec_level_option))
            else exclude.add("record_delim_string")
        )
        (
            include.add("record_length")
            if (self.rec_level_option and "record_length" in str(self.rec_level_option))
            else exclude.add("record_length")
        )
        (
            include.add("record_len_value")
            if (
                ((self.rec_level_option and "record_length" in str(self.rec_level_option)))
                and (self.record_length == " ")
            )
            else exclude.add("record_len_value")
        )
        (
            include.add("record_prefix")
            if (self.rec_level_option and "record_prefix" in str(self.rec_level_option))
            else exclude.add("record_prefix")
        )
        (
            include.add("record_format")
            if (self.rec_level_option and "record_format" in str(self.rec_level_option))
            else exclude.add("record_format")
        )
        (
            include.add("delim")
            if (self.field_option and "delimiter" in str(self.field_option))
            else exclude.add("delim")
        )
        (
            include.add("delim_value")
            if (((self.field_option and "delimiter" in str(self.field_option))) and (self.delim == " "))
            else exclude.add("delim_value")
        )
        include.add("quote") if (self.field_option and "quote" in str(self.field_option)) else exclude.add("quote")
        (
            include.add("quote_value")
            if (((self.field_option and "quote" in str(self.field_option))) and (self.quote == " "))
            else exclude.add("quote_value")
        )
        (
            include.add("actual_length")
            if (self.field_option and "actual_length" in str(self.field_option))
            else exclude.add("actual_length")
        )
        (
            include.add("delim_string")
            if (self.field_option and "delim_string" in str(self.field_option))
            else exclude.add("delim_string")
        )
        (
            include.add("null_length")
            if (self.field_option and "null_length" in str(self.field_option))
            else exclude.add("null_length")
        )
        (
            include.add("null_field")
            if (self.field_option and "null_field" in str(self.field_option))
            else exclude.add("null_field")
        )
        (
            include.add("null_field_sep_flag")
            if (self.field_option and "null_field" in str(self.field_option))
            else exclude.add("null_field_sep_flag")
        )
        (
            include.add("null_field_sep")
            if (
                ((self.null_field_sep_flag))
                and (self.field_option and "null_field" in str(self.field_option))
            )
            else exclude.add("null_field_sep")
        )
        (
            include.add("null_field_sep_value")
            if (
                ((self.field_option and "null_field" in str(self.field_option)))
                and (self.null_field_sep_flag)
                and (self.null_field_sep == " ")
            )
            else exclude.add("null_field_sep_value")
        )
        (
            include.add("prefix")
            if (self.field_option and "prefix_bytes" in str(self.field_option))
            else exclude.add("prefix")
        )
        (
            include.add("print_field")
            if (self.field_option and "print_field" in str(self.field_option))
            else exclude.add("print_field")
        )
        (
            include.add("vector_prefix")
            if (self.field_option and "vector_prefix" in str(self.field_option))
            else exclude.add("vector_prefix")
        )
        (
            include.add("byte_order")
            if (self.general_option and "byte_order" in str(self.general_option))
            else exclude.add("byte_order")
        )
        (
            include.add("charset")
            if (self.general_option and "charset" in str(self.general_option))
            else exclude.add("charset")
        )
        (
            include.add("data_format")
            if (self.general_option and "data_format" in str(self.general_option))
            else exclude.add("data_format")
        )
        (
            include.add("max_width")
            if (self.general_option and "max_width" in str(self.general_option))
            else exclude.add("max_width")
        )
        (
            include.add("width")
            if (self.general_option and "field_width" in str(self.general_option))
            else exclude.add("width")
        )
        (
            include.add("padchar")
            if (self.general_option and "padchar" in str(self.general_option))
            else exclude.add("padchar")
        )
        (
            include.add("padchar_value")
            if (((self.general_option and "padchar" in str(self.general_option))) and (self.padchar == " "))
            else exclude.add("padchar_value")
        )
        (
            include.add("export_ebcdic_as_ascii")
            if (self.string_option and "export_ebcdic_as_ascii" in str(self.string_option))
            else exclude.add("export_ebcdic_as_ascii")
        )
        (
            include.add("import_ascii_as_ebcdic")
            if (self.string_option and "import_ascii_as_ebcdic" in str(self.string_option))
            else exclude.add("import_ascii_as_ebcdic")
        )
        (
            include.add("allow_all_zeros")
            if (self.decimal_options and "allow_all_zeros" in str(self.decimal_options))
            else exclude.add("allow_all_zeros")
        )
        (
            include.add("decimal_separator")
            if (self.decimal_options and "decimal_separator" in str(self.decimal_options))
            else exclude.add("decimal_separator")
        )
        (
            include.add("decimal_sep_value")
            if (
                ((self.decimal_options and "decimal_separator" in str(self.decimal_options)))
                and (self.decimal_separator == " ")
            )
            else exclude.add("decimal_sep_value")
        )
        (
            include.add("decimal_packed")
            if (self.decimal_options and "decimal_packed" in str(self.decimal_options))
            else exclude.add("decimal_packed")
        )
        (
            include.add("precision")
            if (self.decimal_options and "precision" in str(self.decimal_options))
            else exclude.add("precision")
        )
        (
            include.add("round")
            if (self.decimal_options and "round" in str(self.decimal_options))
            else exclude.add("round")
        )
        (
            include.add("scale")
            if (self.decimal_options and "scale" in str(self.decimal_options))
            else exclude.add("scale")
        )
        (
            include.add("decimal_packed_check")
            if (
                ((self.decimal_options and "decimal_packed" in str(self.decimal_options)))
                and (self.decimal_packed == "packed")
            )
            else exclude.add("decimal_packed_check")
        )
        (
            include.add("decimal_packed_sign_position")
            if (
                ((self.decimal_options and "decimal_packed" in str(self.decimal_options)))
                and (self.decimal_packed != "packed")
            )
            else exclude.add("decimal_packed_sign_position")
        )
        (
            include.add("decimal_packed_signed")
            if (
                ((self.decimal_options and "decimal_packed" in str(self.decimal_options)))
                and (((self.decimal_packed == "packed")) or (self.decimal_packed == "zoned"))
            )
            else exclude.add("decimal_packed_signed")
        )
        (
            include.add("allow_signed_import")
            if (
                ((self.decimal_options and "decimal_packed" in str(self.decimal_options)))
                and (self.decimal_packed == "packed")
                and (self.decimal_packed_signed == "unsigned")
            )
            else exclude.add("allow_signed_import")
        )
        (
            include.add("c_format")
            if (self.numeric_option and "c_format" in str(self.numeric_option))
            else exclude.add("c_format")
        )
        (
            include.add("in_format")
            if (self.numeric_option and "in_format" in str(self.numeric_option))
            else exclude.add("in_format")
        )
        (
            include.add("out_format")
            if (self.numeric_option and "out_format" in str(self.numeric_option))
            else exclude.add("out_format")
        )
        include.add("days_since") if (self.date_option == "days_since") else exclude.add("days_since")
        include.add("date_format") if (self.date_option == "date_format") else exclude.add("date_format")
        include.add("is_julian") if (self.date_option == "is_julian") else exclude.add("is_julian")
        include.add("time_format") if (self.time_option == "time_format") else exclude.add("time_format")
        (
            include.add("is_midnight_seconds")
            if (self.time_option == "is_midnight_seconds")
            else exclude.add("is_midnight_seconds")
        )
        (
            include.add("timestamp_format")
            if (self.timestamp_option == "timestamp_format")
            else exclude.add("timestamp_format")
        )
        return include, exclude

    def get_input_ports_props(self) -> dict:
        include, exclude = self.validate()
        props = {"runtime_column_propagation"}
        required = set()
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property: {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties: {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_input_cardinality(self) -> dict:
        return {"min": 1, "max": 1}

    def get_output_ports_props(self) -> dict:
        include, exclude = self.validate()
        props = {"field", "schema_"}
        required = {"field", "schema_"}
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property: {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties: {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_output_cardinality(self) -> dict:
        return {"min": 1, "max": 1}

    def get_parameters_props(self) -> dict:
        include, exclude = self.validate()
        props = {
            "actual_length",
            "allow_all_zeros",
            "allow_signed_import",
            "auto_column_propagation",
            "buf_free_run",
            "buf_free_run_ronly",
            "buf_mode",
            "buf_mode_ronly",
            "byte_order",
            "c_format",
            "charset",
            "check_intact",
            "check_intact_flag",
            "coll_type",
            "combinability",
            "current_output_link_type",
            "data_format",
            "date_format",
            "date_option",
            "days_since",
            "decimal_options",
            "decimal_packed",
            "decimal_packed_check",
            "decimal_packed_sign_position",
            "decimal_packed_signed",
            "decimal_sep_value",
            "decimal_separator",
            "delim",
            "delim_string",
            "delim_value",
            "disk_write_inc",
            "disk_write_inc_ronly",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "execmode",
            "export_ebcdic_as_ascii",
            "field",
            "field_option",
            "fill",
            "fill_char_value",
            "final_delim",
            "final_delim_string",
            "final_delim_value",
            "flow_dirty",
            "general_option",
            "hide",
            "import_ascii_as_ebcdic",
            "in_format",
            "input_count",
            "input_link_description",
            "inputcol_properties",
            "intact",
            "is_julian",
            "is_midnight_seconds",
            "keep_field",
            "key_col_select",
            "key_cols_coll",
            "key_cols_coll_ordered",
            "key_cols_coll_robin",
            "key_cols_none",
            "key_cols_part",
            "max_mem_buf_size",
            "max_mem_buf_size_ronly",
            "max_width",
            "null_field",
            "null_field_sep",
            "null_field_sep_flag",
            "null_field_sep_value",
            "null_length",
            "numeric_option",
            "out_format",
            "output_acp_should_hide",
            "output_count",
            "output_link_description",
            "outputcol_properties",
            "padchar",
            "padchar_value",
            "part_client_dbname",
            "part_client_instance",
            "part_dbconnection",
            "part_stable",
            "part_stable_coll",
            "part_stable_ordered",
            "part_stable_roundrobin_coll",
            "part_table",
            "part_type",
            "part_unique",
            "part_unique_coll",
            "part_unique_ordered",
            "part_unique_roundrobin_coll",
            "perform_sort",
            "perform_sort_coll",
            "perform_sort_modulus",
            "precision",
            "prefix",
            "preserve",
            "print_field",
            "queue_upper_size",
            "queue_upper_size_ronly",
            "quote",
            "quote_value",
            "rec_level_option",
            "record_delim",
            "record_delim_string",
            "record_delim_value",
            "record_format",
            "record_len_value",
            "record_length",
            "record_prefix",
            "round",
            "runtime_column_propagation",
            "save_rejects_fail_rejects",
            "scale",
            "schema_",
            "schemafile",
            "selection",
            "show_coll_type",
            "show_part_type",
            "show_sort_options",
            "sort_instructions",
            "sort_instructions_text",
            "stage_description",
            "string_option",
            "time_format",
            "time_option",
            "timestamp_format",
            "timestamp_option",
            "vector_prefix",
            "width",
        }
        required = {
            "current_output_link_type",
            "enable_flow_acp_control",
            "enable_schemaless_design",
            "field",
            "output_acp_should_hide",
            "schema_",
            "schemafile",
            "selection",
        }
        props = props & (self.model_fields_set | required)
        conflict = exclude & self.model_fields_set & props
        if conflict:
            conflict = list(conflict)
            if len(conflict) == 1:
                warnings.warn(f"\n\033[33mFound conflicting property {conflict[0]}\033[0m")
            else:
                warnings.warn(
                    f'\n\033[33mFound conflicting properties {", ".join(conflict[:-1])} and {conflict[-1]}\033[0m'
                )
        exclude = exclude - self.model_fields_set
        return self.model_dump(include=props - exclude, by_alias=True, exclude_none=True, warnings=False)

    def get_app_data_props(self) -> dict:
        return {
            "datastage": {
                "maxRejectOutputs": 1,
                "minRejectOutputs": 0,
                "maxReferenceInputs": 0,
                "minReferenceInputs": 0,
            }
        }
